--[[
	* Copyright (c) 2020 by Antoine Desmarets.
	* Cixi of Remulos Oceanic / WoW Classic
	*
	* EarlyThreat is distributed in the hope that it will be useful and entertaining,
	* but WITHOUT ANY WARRANTY
]]--


local etlocal_version = "1.13.5.006"  --change here, and in TOC
local etlocal_reqPrefix = "EarlyThreat;"
local etlocal_unitName = ""
local etlocal_ticker -- ticker for combat duration check
local etlocal_elapsed -- seconds elapsed since start of combat

local _G = getfenv(0)


local etlocal_threat =	 	{	{ "shieldslam", 	"Shield Slam",		true  },
								{ "sunderarmor", 	"Sunder Armor",		true  },
								{ "maul", 			"Maul",				true  },
								{ "heroicstrike", 	"Heroic Strike",	false },
								{ "revenge", 		"Revenge",			false },
								{ "swipe", 			"Swipe",			false },
								{ "bloodthirst", 	"Bloodthirst", 		false },
							}

local etlocal_threatwarn = 	{ 	{ "s",				false },
								{ "y",				true  },
								{ "p",				false },
								{ "rw",				false },
							}
local etlocal_disableafter = 15


local etlocal_taunt = 		{ 	{ "taunt", 			"Taunt",			true  },
								{ "mockingblow", 	"Mocking Blow",		false },
								{ "growl", 			"Growl",			true  },
							}

local etlocal_tauntok = 	{	{ "s",				true  },
								{ "y",				false },
								{ "p",				false },
								{ "rw",				false },
							}
local etlocal_tauntwarn =	{ 	{ "s",				false },
								{ "y",				true  },
								{ "p",				false },
								{ "rw",				false },
							}


local etlocal_misstypes = 	{ 	{ "ABSORB", 		false,		"ABSORBED"	 },
								{ "BLOCK", 			true,		"BLOCKED"	 },
								{ "DEFLECT", 		false,		"DEFLECTED"	 },
								{ "DODGE", 			true,		"DODGED"	 },
								{ "EVADE", 			false,		"EVADED"	 },
								{ "IMMUNE", 		false,		"IMMUNE"	 },
								{ "MISS", 			true,		"MISSED"	 },
								{ "PARRY", 			true,		"PARRIED"	 },
								{ "REFLECT", 		false,		"REFLECTED"	 },
								{ "RESIST", 		true, 		"RESISTED" 	 },
							}

local etlocal_chatTypes = 	{ 	s = "SAY",
								y = "YELL",
								p = "PARTY",
								r = "RAID",
								rw = "RAID_WARNING"
							}
local 	raidIcons = 		{
								[COMBATLOG_OBJECT_RAIDTARGET1] = "{RT1}",
								[COMBATLOG_OBJECT_RAIDTARGET2] = "{RT2}",
								[COMBATLOG_OBJECT_RAIDTARGET3] = "{RT3}",
								[COMBATLOG_OBJECT_RAIDTARGET4] = "{RT4}",
								[COMBATLOG_OBJECT_RAIDTARGET5] = "{RT5}",
								[COMBATLOG_OBJECT_RAIDTARGET6] = "{RT6}",
								[COMBATLOG_OBJECT_RAIDTARGET7] = "{RT7}",
								[COMBATLOG_OBJECT_RAIDTARGET8] = "{RT8}",
							}							
-------------------------------------------------------------------------

function EarlyThreat_OnLoad()

	EarlyThreat_ContainerFrame:RegisterEvent("ADDON_LOADED");					--Initialisation
	EarlyThreat_ContainerFrame:RegisterEvent("PLAYER_ENTERING_WORLD");			--Toon Initialisation	
	EarlyThreat_ContainerFrame:RegisterEvent("PLAYER_REGEN_DISABLED");			--Detect start of combat
	EarlyThreat_ContainerFrame:RegisterEvent("PLAYER_REGEN_ENABLED");			--Detect out of combat
	
	EarlyThreat_ContainerFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")	

end

-------------------------------------------------------------------------

function EarlyThreat_ToggleWarnings()
	if EarlyThreat_Enabled then
		EarlyThreat_Enabled = false
		print("EarlyThreat: Warnings are now OFF")
	else 
		EarlyThreat_Enabled = true
		print("EarlyThreat: Warnings are now ON")
	end
	_G["EarlyThreat_ContainerFrame_toggle"]:SetChecked(EarlyThreat_Enabled)
end 

-------------------------------------------------------------------------

function EarlyThreat_OnEvent(event, ...)
	local arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg, arg9 = ...
	
	if event == "ADDON_LOADED" and arg1 == "EarlyThreat" then
		DEFAULT_CHAT_FRAME:AddMessage("EarlyThreat v."..etlocal_version.." by Cixi@Remulos. Type /EarlyThreat or /wr to get started,")
		--DEFAULT_CHAT_FRAME:AddMessage("or use the minimap button to toggle the app.")


		if (EarlyThreat_Settings == nil) then EarlyThreat_Settings = {}; end
		if (EarlyThreat_Settings["threat"] == nil) then 
			EarlyThreat_Settings["threat"] = etlocal_threat;	
			EarlyThreat_Settings["threatwarn"] = etlocal_threatwarn;	
			EarlyThreat_Settings["disableafter"] = etlocal_disableafter;	
			EarlyThreat_Settings["taunt"] = etlocal_taunt;	
			EarlyThreat_Settings["tauntok"] = etlocal_tauntok;	
			EarlyThreat_Settings["tauntwarn"] = etlocal_tauntwarn;	
			EarlyThreat_Settings["misstypes"] = etlocal_misstypes
		end

		if EarlyThreat_Enabled == nil then EarlyThreat_Enabled = true end

		_G["EarlyThreat_ContainerFrame_version"]:SetText("Version " .. etlocal_version);

		--update checkbox names and set default values
		_G["EarlyThreat_ContainerFrame_shieldslamText"]:SetText("        Shield Slam")
		_G["EarlyThreat_ContainerFrame_sunderarmorText"]:SetText("        Sunder Armor")
		_G["EarlyThreat_ContainerFrame_bloodthirstText"]:SetText("        Bloodthirst")
		_G["EarlyThreat_ContainerFrame_maulText"]:SetText("        Maul")
		_G["EarlyThreat_ContainerFrame_heroicstrikeText"]:SetText("        Heroic Strike")
		_G["EarlyThreat_ContainerFrame_revengeText"]:SetText("        Revenge")
		_G["EarlyThreat_ContainerFrame_swipeText"]:SetText("        Swipe")
		for id, spellpair in pairs(EarlyThreat_Settings["threat"]) do
			_G["EarlyThreat_ContainerFrame_"..spellpair[1]]:SetChecked(spellpair[3])
		end 

		_G["EarlyThreat_ContainerFrame_threatsText"]:SetText("Say")
		_G["EarlyThreat_ContainerFrame_threatyText"]:SetText("Yell")
		_G["EarlyThreat_ContainerFrame_threatpText"]:SetText("Raid/Party")
		_G["EarlyThreat_ContainerFrame_threatrwText"]:SetText("Raid Warning")
		for id, spellpair in pairs(EarlyThreat_Settings["threatwarn"]) do
			_G["EarlyThreat_ContainerFrame_threat"..spellpair[1]]:SetChecked(spellpair[2])
		end 

		_G["EarlyThreat_ContainerFrame_tauntText"]:SetText("        Taunt")
		_G["EarlyThreat_ContainerFrame_mockingblowText"]:SetText("        Mocking Blow")
		_G["EarlyThreat_ContainerFrame_growlText"]:SetText("        Growl")
		for id, spellpair in pairs(EarlyThreat_Settings["taunt"]) do
			_G["EarlyThreat_ContainerFrame_"..spellpair[1]]:SetChecked(spellpair[3])
		end 

		_G["EarlyThreat_ContainerFrame_oksText"]:SetText("Say")
		_G["EarlyThreat_ContainerFrame_okyText"]:SetText("Yell")
		_G["EarlyThreat_ContainerFrame_okpText"]:SetText("Raid/Party")
		_G["EarlyThreat_ContainerFrame_okrwText"]:SetText("Raid Warning")
		for id, spellpair in pairs(EarlyThreat_Settings["tauntok"]) do
			_G["EarlyThreat_ContainerFrame_ok"..spellpair[1]]:SetChecked(spellpair[2])
		end 

		_G["EarlyThreat_ContainerFrame_resistsText"]:SetText("Say")
		_G["EarlyThreat_ContainerFrame_resistyText"]:SetText("Yell")
		_G["EarlyThreat_ContainerFrame_resistpText"]:SetText("Raid/Party")
		_G["EarlyThreat_ContainerFrame_resistrwText"]:SetText("Raid Warning")
		for id, spellpair in pairs(EarlyThreat_Settings["tauntwarn"]) do
			_G["EarlyThreat_ContainerFrame_resist"..spellpair[1]]:SetChecked(spellpair[2])
		end 
		
		_G["EarlyThreat_ContainerFrame_disableafter"]:SetText(EarlyThreat_Settings["disableafter"])

		_G["EarlyThreat_ContainerFrame_toggle"]:SetChecked(EarlyThreat_Enabled)
	end
	

	if event == "PLAYER_ENTERING_WORLD" then
		etlocal_unitName = UnitName("player")
	end


	if event == "PLAYER_REGEN_DISABLED" then
		etlocal_elapsed = 0
		etlocal_disregard = false
		etlocal_ticker = C_Timer.NewTicker(1, EarlyThreat_DisregardEvents )
	end

	if event == "PLAYER_REGEN_ENABLED" then
		--print("OOC")
		etlocal_elapsed = 0
		etlocal_disregard = true
		etlocal_ticker:Cancel()
	end


	if event == "COMBAT_LOG_EVENT_UNFILTERED" then
		inInstance, instanceType = IsInInstance()
		if instanceType == "party" or instanceType == "raid" then 
			EarlyThreat_CheckResist(event)
		end
	end

end

-------------------------------------------------------------------------

function EarlyThreat_DisregardEvents()

	etlocal_elapsed = etlocal_elapsed + 1
	--print(etlocal_elapsed)
	if etlocal_elapsed >= EarlyThreat_Settings["disableafter"] then   
		etlocal_disregard = true
		etlocal_ticker:Cancel()
		inInstance, instanceType = IsInInstance()
		if (instanceType == "party" or instanceType == "raid") and EarlyThreat_Enabled then 
			print("EarlyThreat: Disregarding missed attacks for the rest of the fight")
		end
	end

end

-------------------------------------------------------------------------

function EarlyThreat_CheckResist(event)
	local timestamp, subevent, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, 
	destGUID, destName, destFlags, destRaidFlags, spellId, spellName, spellSchool, 
	arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23, arg24  = CombatLogGetCurrentEventInfo()
	

	if EarlyThreat_Enabled then
	
		local icon = (raidIcons[bit.band(destRaidFlags, COMBATLOG_OBJECT_RAIDTARGET_MASK)] or "")
		if icon ~= "" then icon = icon .. " " end 

		if sourceName == etlocal_unitName then 

			-- SUCCESSFUL TAUNT
			if subevent == "SPELL_CAST_SUCCESS" then
				
				-- Checking it's one of the spells we're monitoring
				for id, spellpair in pairs(EarlyThreat_Settings["taunt"]) do
					if spellName == spellpair[2] and spellpair[3] == true then 

						-- Checking which channel to throw the message in
						for idw, okpair in pairs(EarlyThreat_Settings["tauntok"]) do
							if okpair[2] == true then 

								local channel = okpair[1]
								if channel == 'p' then if UnitInRaid("player") then channel = 'r' end end
								SendChatMessage(string.upper(spellName) .. " on " .. icon .. destName, etlocal_chatTypes[channel])
							end 
						end

					end 	
				end
				
			end

			-- FAILED TAUNT
			if subevent == "SPELL_MISSED" then
				local _, _, prefix, suffix = string.find (subevent, "(.-)_(.+)")

				-- Checking it's one of the spells we're monitoring
				for id, spellpair in pairs(EarlyThreat_Settings["taunt"]) do
					if spellName == spellpair[2] and spellpair[3] == true then 

	--					if arg15 ~= nil then --checking miss type

							-- Checking which channel to throw the message in
							for idw, warnpair in pairs(EarlyThreat_Settings["tauntwarn"]) do
								if warnpair[2] == true then 

									-- Checking it's one of the miss types we want
	--								for idm, typepair in pairs(EarlyThreat_Settings["misstypes"]) do
	--									if typepair[1] == arg15 and typepair[2] == true then 


											local channel = warnpair[1]
											if channel == 'p' then if UnitInRaid("player") then channel = 'r' end end
											SendChatMessage(string.upper(spellName).." FAILED on " .. icon .. destName .. "!", etlocal_chatTypes[channel])
	--									end
	--								end 
								end 
							end
	--					end 
					end 	
				end
			end

			-- FAILED THREAT
			if subevent == "SPELL_MISSED" and etlocal_disregard == false then
				local _, _, prefix, suffix = string.find (subevent, "(.-)_(.+)")

				-- Checking it's one of the spells we're monitoring
				for id, spellpair in pairs(EarlyThreat_Settings["threat"]) do
					if spellName == spellpair[2] and spellpair[3] == true then 
						if arg15 ~= nil then
				
							-- Checking which channel to throw the message in
							for idw, warnpair in pairs(EarlyThreat_Settings["threatwarn"]) do
								if warnpair[2] == true then 

									-- Checking it's one of the miss types we want
									for idm, typepair in pairs(EarlyThreat_Settings["misstypes"]) do
										if typepair[1] == arg15 and typepair[2] == true then 

											local channel = warnpair[1]
											if channel == 'p' then if UnitInRaid("player") then channel = 'r' end end
											SendChatMessage(spellName .. " on " .. icon .. destName .. " has been " .. typepair[3] .. "! Mind your threat!", etlocal_chatTypes[channel])
										end
									end
								end 
							end
						end 
					end 	
				end
			end		

		end
	end
end
 
-------------------------------------------------------------------------

function EarlyThreat_SetCheckedWarn(arr, which, checked)

	for id, pair in pairs(EarlyThreat_Settings[arr]) do
		if pair[1] == which then 
			pair[2] = checked
		end
	end
	
end

-------------------------------------------------------------------------

function EarlyThreat_SetCheckedSpell(arr, which, checked)
 	local etlocal_newability = true

	for id, pair in pairs(EarlyThreat_Settings[arr]) do
		if pair[1] == which then 
			pair[3] = checked
			etlocal_newability = false
		end
	end
	
	-- add potential new abilities in updated releases
	if etlocal_newability then 
		for id, pair in pairs(etlocal_threat) do
			if pair[1] == which then
				pair[3] = checked  --change defautl value to be selection
				table.insert(EarlyThreat_Settings[arr], pair)
			end
		end
	end

end

-------------------------------------------------------------------------

function EarlyThreat_SetEditbox(box, field)
	box:SetTextColor(1, 1, 1)
	local value = tonumber(box:GetNumber())
	if value and value > 0 then
		EarlyThreat_Settings["disableafter"] = value
--		print("update to ".. value)
		box:ClearFocus()
	end
end


-------------------------------------------------------------------------

function EarlyThreat_split(str, sep)
	local t = {}
	local ind = string.find(str, sep)
	while (ind ~= nil) do 	
		table.insert(t, string.sub(str, 1, ind-1))
		str = string.sub(str, ind+1)
		ind = string.find(str, sep, 1, true)
	end
	if (str ~="") then table.insert(t, str) end
	return t
end

-------------------------------------------------------------------------

function EarlyThreat_SlashCommandHandler( msg )
	local args = EarlyThreat_split(msg, " ")
	if args[1] == "toggle" then 
		EarlyThreat_ToggleWarnings()
	else 
		EarlyThreat_ContainerFrame:Show()
	end
end

-------------------------------------------------------------------------

SlashCmdList["EarlyThreat"] = EarlyThreat_SlashCommandHandler
SLASH_EarlyThreat1 = "/EarlyThreat"
SLASH_EarlyThreat2 = "/et"

